@@ -6,12 +6,12 @@ use std::process;
 pub fn cli() -> App {
     subcommand("verify-project")
         .about("Check correctness of crate manifest")
-        .arg(opt("quiet", "Do not print cargo log messages").short("q"))
+        .arg_quiet()
         .arg_manifest_path()
         .after_help("Run `cargo help verify-project` for more detailed information.\n")
 }
 
-pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
+pub fn exec(config: &mut Config, args: &ArgMatches) -> CliResult {
     if let Err(e) = args.workspace(config) {
         let mut h = HashMap::new();
         h.insert("invalid".to_string(), e.to_string());
