@@ -50,7 +50,7 @@
 //! improved.
 
 use std::cell::{Cell, RefCell};
-use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
+use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fmt::Write as _;
 use std::io;
 use std::marker;
@@ -61,7 +61,7 @@ use anyhow::{format_err, Context as _};
 use cargo_util::ProcessBuilder;
 use crossbeam_utils::thread::Scope;
 use jobserver::{Acquired, Client, HelperThread};
-use log::{debug, info, trace};
+use log::{debug, trace};
 use semver::Version;
 
 use super::context::OutputFile;
@@ -72,7 +72,7 @@ use super::job::{
 use super::timings::Timings;
 use super::{BuildContext, BuildPlan, CompileMode, Context, Unit};
 use crate::core::compiler::future_incompat::{
-    FutureBreakageItem, FutureIncompatReportPackage, OnDiskReports,
+    self, FutureBreakageItem, FutureIncompatReportPackage,
 };
 use crate::core::resolver::ResolveBehavior;
 use crate::core::{PackageId, Shell, TargetKind};
@@ -413,7 +413,7 @@ impl<'cfg> JobQueue<'cfg> {
 
         // This is somewhat tricky, but we may need to synthesize some
         // dependencies for this target if it requires full upstream
-        // compilations to have completed. If we're in pipelining mode then some
+        // compilations to have completed. Because of pipelining, some
         // dependency edges may be `Metadata` due to the above clause (as
         // opposed to everything being `All`). For example consider:
         //
@@ -649,7 +649,7 @@ impl<'cfg> DrainState<'cfg> {
                     // If `id` has completely finished we remove it
                     // from the `active` map ...
                     Artifact::All => {
-                        info!("end: {:?}", id);
+                        trace!("end: {:?}", id);
                         self.finished += 1;
                         if let Some(rustc_tokens) = self.rustc_tokens.remove(&id) {
                             // This puts back the tokens that this rustc
@@ -670,11 +670,11 @@ impl<'cfg> DrainState<'cfg> {
                     // ... otherwise if it hasn't finished we leave it
                     // in there as we'll get another `Finish` later on.
                     Artifact::Metadata => {
-                        info!("end (meta): {:?}", id);
+                        trace!("end (meta): {:?}", id);
                         self.active[&id].clone()
                     }
                 };
-                info!("end ({:?}): {:?}", unit, result);
+                debug!("end ({:?}): {:?}", unit, result);
                 match result {
                     Ok(()) => self.finish(id, &unit, artifact, cx)?,
                     Err(e) => {
@@ -695,7 +695,7 @@ impl<'cfg> DrainState<'cfg> {
                 self.tokens.push(token);
             }
             Message::NeedsToken(id) => {
-                log::info!("queue token request");
+                trace!("queue token request");
                 jobserver_helper.request_token();
                 let client = cx.rustc_clients[&self.active[&id]].clone();
                 self.to_send_clients
@@ -733,7 +733,7 @@ impl<'cfg> DrainState<'cfg> {
         // listen for a message with a timeout, and on timeout we run the
         // previous parts of the loop again.
         let mut events = self.messages.try_pop_all();
-        info!(
+        trace!(
             "tokens in use: {}, rustc_tokens: {:?}, waiting_rustcs: {:?} (events this tick: {})",
             self.tokens.len(),
             self.rustc_tokens
@@ -839,7 +839,7 @@ impl<'cfg> DrainState<'cfg> {
         }
 
         let time_elapsed = util::elapsed(cx.bcx.config.creation_time().elapsed());
-        if let Err(e) = self.timings.finished(cx.bcx, &error) {
+        if let Err(e) = self.timings.finished(cx, &error) {
             if error.is_some() {
                 crate::display_error(&e, &mut cx.bcx.config.shell());
             } else {
@@ -871,7 +871,10 @@ impl<'cfg> DrainState<'cfg> {
             if !cx.bcx.build_config.build_plan {
                 // It doesn't really matter if this fails.
                 drop(cx.bcx.config.shell().status("Finished", message));
-                self.emit_future_incompat(cx.bcx);
+                future_incompat::save_and_display_report(
+                    cx.bcx,
+                    &self.per_package_future_incompat_reports,
+                );
             }
 
             None
@@ -881,76 +884,6 @@ impl<'cfg> DrainState<'cfg> {
         }
     }
 
-    fn emit_future_incompat(&mut self, bcx: &BuildContext<'_, '_>) {
-        if !bcx.config.cli_unstable().future_incompat_report {
-            return;
-        }
-        let should_display_message = match bcx.config.future_incompat_config() {
-            Ok(config) => config.should_display_message(),
-            Err(e) => {
-                crate::display_warning_with_error(
-                    "failed to read future-incompat config from disk",
-                    &e,
-                    &mut bcx.config.shell(),
-                );
-                true
-            }
-        };
-
-        if self.per_package_future_incompat_reports.is_empty() {
-            // Explicitly passing a command-line flag overrides
-            // `should_display_message` from the config file
-            if bcx.build_config.future_incompat_report {
-                drop(
-                    bcx.config
-                        .shell()
-                        .note("0 dependencies had future-incompatible warnings"),
-                );
-            }
-            return;
-        }
-
-        // Get a list of unique and sorted package name/versions.
-        let package_vers: BTreeSet<_> = self
-            .per_package_future_incompat_reports
-            .iter()
-            .map(|r| r.package_id)
-            .collect();
-        let package_vers: Vec<_> = package_vers
-            .into_iter()
-            .map(|pid| pid.to_string())
-            .collect();
-
-        if should_display_message || bcx.build_config.future_incompat_report {
-            drop(bcx.config.shell().warn(&format!(
-                "the following packages contain code that will be rejected by a future \
-                 version of Rust: {}",
-                package_vers.join(", ")
-            )));
-        }
-
-        let on_disk_reports =
-            OnDiskReports::save_report(bcx.ws, &self.per_package_future_incompat_reports);
-        let report_id = on_disk_reports.last_id();
-
-        if bcx.build_config.future_incompat_report {
-            let rendered = on_disk_reports.get_report(report_id, bcx.config).unwrap();
-            drop(bcx.config.shell().print_ansi_stderr(rendered.as_bytes()));
-            drop(bcx.config.shell().note(&format!(
-                "this report can be shown with `cargo report \
-                 future-incompatibilities -Z future-incompat-report --id {}`",
-                report_id
-            )));
-        } else if should_display_message {
-            drop(bcx.config.shell().note(&format!(
-                "to see what the problems were, use the option \
-                 `--future-incompat-report`, or run `cargo report \
-                 future-incompatibilities --id {}`",
-                report_id
-            )));
-        }
-    }
-
     fn handle_error(
         &self,
         shell: &mut Shell,
@@ -973,7 +906,7 @@ impl<'cfg> DrainState<'cfg> {
     // this as often as we spin on the events receiver (at least every 500ms or
     // so).
     fn tick_progress(&mut self) {
-        // Record some timing information if `-Ztimings` is enabled, and
+        // Record some timing information if `--timings` is enabled, and
         // this'll end up being a noop if we're not recording this
         // information.
         self.timings.mark_concurrency(
@@ -1033,7 +966,7 @@ impl<'cfg> DrainState<'cfg> {
         let id = JobId(self.next_id);
         self.next_id = self.next_id.checked_add(1).unwrap();
 
-        info!("start {}: {:?}", id, unit);
+        debug!("start {}: {:?}", id, unit);
 
         assert!(self.active.insert(id, unit.clone()).is_none());
 
